
/*
    xsit - screen mate for X11
    Copyright (C) 2003 Jrme Laheurte <fraca7@free.fr>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* $Id: animationsitter.c,v 1.4 2003/08/04 14:40:16 jerome Exp $ */

#include <Imlib.h>

#include "animationsitter.h"
#include "utils.h"

extern ImlibData *XsitImlibData;

typedef struct {
  sitter super;           /* c'est pas b l'hritage en C, hein ? */

  GSList *frames;
  GSList *current_frame;
  GSList *delays;
  GSList *current_delay;
  geometry *geom;
} animation_sitter;

void animation_sitter_next_frame(sitter *s) {
  animation_sitter *st = (animation_sitter*)s;

  st->current_frame = g_slist_next(st->current_frame);
  st->current_delay = g_slist_next(st->current_delay);
  if (st->current_frame == NULL) {
    st->current_frame = st->frames;
    st->current_delay = st->delays;
  }
}

ImlibImage *animation_sitter_get_current_frame(sitter *s) {
  animation_sitter *st = (animation_sitter*)s;
  return (ImlibImage*)(st->current_frame->data);
}

gint animation_sitter_get_next_timeout(sitter *s) {
  animation_sitter *st = (animation_sitter*)s;
  return GPOINTER_TO_INT(st->current_delay->data);
}

gint animation_sitter_get_x(sitter *s, gint x, gint y, guint w, guint h) {
  return geometry_get_x(((animation_sitter*)s)->geom, w, s->w) + x;
}

gint animation_sitter_get_y(sitter *s, gint x, gint y, guint w, guint h) {
  return geometry_get_y(((animation_sitter*)s)->geom, h, s->h) + y;
}

sitter *animation_sitter_new(gchar *str) {
  animation_sitter *st;
  gchar *path;
  gint delay;
  ImlibImage *img;
  token_string *tk = token_string_new(str, ':');

  if (!token_string_has_more(tk)) {
    fprintf(stderr, "Parse error:\n%s\n", str);
    exit(1);
  }

  st = g_new(animation_sitter, 1);
  st->super.type = ANIMATION_SITTER;

  if ((st->geom = geometry_new(token_string_next(tk))) == NULL) {
    token_string_free(tk);
    g_free(st);
    return NULL;
  }

  st->frames = st->current_frame = st->delays = NULL;
  while (token_string_has_more(tk)) {
    path = token_string_next(tk);
    if (!token_string_has_more(tk)) {

      /* Une petite exception ici,  cause des GList... Mieux vaut se vautrer
	 que provoquer une fuite de mmoire... Bonjour  toute l'quipe de
	 dveloppement de Windows :)
	 En fait, je n'ai pas envie de me faire ch...  librer toute la liste
	 mais crire ce commentaire m'a pris autant de temps qu'il m'en aurait
	 fallu pour le faire. Dura lex, sed lex. */

      fprintf(stderr, "Parse error in ~/.xsitrc:\n%s\n", str);
      exit(1);
    }
    if (sscanf(token_string_next(tk), "%d", &delay) != 1) {
      fprintf(stderr, "Parse error in ~/.xsitrc:\n%s\n", str);
      exit(1);
    }

    st->delays = g_slist_append(st->delays, GINT_TO_POINTER(delay));
    img = Imlib_load_image(XsitImlibData, path);
    if (img == NULL) {
      fprintf(stderr, "Cannot load image %s\n", path);
      exit(1);
    }
    Imlib_render(XsitImlibData,
		 img,
		 img->rgb_width,
		 img->rgb_height);
    st->frames = g_slist_append(st->frames, img);
  }

  st->current_frame = st->frames;
  st->current_delay = st->delays;

  st->super.w = ((ImlibImage*)(st->frames->data))->rgb_width;
  st->super.h = ((ImlibImage*)(st->frames->data))->rgb_height;

  st->super.next_frame = animation_sitter_next_frame;
  st->super.get_current_frame = animation_sitter_get_current_frame;
  st->super.get_next_timeout = animation_sitter_get_next_timeout;
  st->super.get_x = animation_sitter_get_x;
  st->super.get_y = animation_sitter_get_y;
  st->super.structure_notify = NULL;
  st->super.focus_notify = NULL;

  return (sitter*)st;
}

void animation_sitter_free(sitter *s) {
  GSList *l;
  animation_sitter *st = (animation_sitter*)s;

  l = st->frames;
  while (l != NULL) {
    Imlib_destroy_image(XsitImlibData, (ImlibImage*)(l->data));
    l = g_slist_next(l);
  }
  g_slist_free(st->frames);
  g_slist_free(st->delays);
  geometry_free(st->geom);
  sitter_free(s);
}

static const char* _rcsid_animationsitter __attribute__((unused)) = "$Id: animationsitter.c,v 1.4 2003/08/04 14:40:16 jerome Exp $";
