
/*
    xsit - screen mate for X11
    Copyright (C) 2003 Jrme Laheurte <fraca7@free.fr>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* $Id: geom.c,v 1.4 2003/08/04 14:40:16 jerome Exp $ */

#include <stdio.h>

#include "geom.h"
#include "utils.h"

geometry *geometry_new(gchar *l) {
  geometry *g;
  gchar c;
  gint pos;
  gchar *str;

  token_string *tk = token_string_new(l, '/');
  if (!token_string_has_more(tk)) {
    token_string_free(tk);
    return NULL;
  }

  str = token_string_next(tk);
  if (!token_string_has_more(tk)) {
    token_string_free(tk);
    return NULL;
  }

  g = g_new(geometry, 1);

  if ((str[0] == '-') || (str[0] == '+')) {
    sscanf(str, "%c%d", &c, &pos);
    g->isXRel = (c == '+') ? -1 : 1;
    g->xVal = pos;
  } else if (sscanf(str, "%d%%", &pos) == 1) {
    g->isXRel = 0;
    g->xVal = pos;
  } else {
    token_string_free(tk);
    g_free(g);
    return NULL;
  }

  str = token_string_next(tk);

  if ((str[0] == '-') || (str[0] == '+')) {
    sscanf(str, "%c%d", &c, &pos);
    g->isYRel = (c == '+') ? -1 : 1;
    g->yVal = pos;
  } else if (sscanf(str, "%d%%", &pos) == 1) {
    g->isYRel = 0;
    g->yVal = pos;
  } else {
    token_string_free(tk);
    g_free(g);
    return NULL;
  }

  token_string_free(tk);
  return g;
}

gint geometry_get_coord(gint isRel,
			gint val,
			guint dimw,
			guint dims) {
  gint r;

  if (isRel == 0) {
    r = (dimw-dims)*val/100;
  } else if (isRel == 1) {
    r = dimw-val;
  } else {
    r = val;
  }
  return r;
}

gint geometry_get_x(geometry *g, guint ww, guint sw) {
  return geometry_get_coord(g->isXRel, g->xVal, ww, sw);
}

gint geometry_get_y(geometry *g, guint wh, guint sh) {
  return geometry_get_coord(g->isYRel, g->yVal, wh, sh);
}

static const char* _rcsid_geom __attribute__((unused)) = "$Id: geom.c,v 1.4 2003/08/04 14:40:16 jerome Exp $";
