
/*
    xsit - screen mate for X11
    Copyright (C) 2003 Jrme Laheurte <fraca7@free.fr>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* $Id: imagesitter.c,v 1.4 2003/08/04 14:40:16 jerome Exp $ */

#include <Imlib.h>

#include "imagesitter.h"
#include "utils.h"

extern ImlibData *XsitImlibData;

typedef struct {
  sitter super;
  ImlibImage *img;
  geometry *geom;
} image_sitter;

void image_sitter_next_frame(sitter *s) {
}

ImlibImage *image_sitter_get_current_frame(sitter *s) {
  return ((image_sitter*)s)->img;
}

gint image_sitter_get_next_timeout(sitter *s) {
  return 0;
}

gint image_sitter_get_x(sitter *s, gint x, gint y, guint w, guint h) {
  return geometry_get_x(((image_sitter*)s)->geom, w, s->w) + x;
}

gint image_sitter_get_y(sitter *s, gint x, gint y, guint w, guint h) {
  return geometry_get_y(((image_sitter*)s)->geom, h, s->h) + y;
}

sitter *image_sitter_new(gchar *line) {
  token_string *tk;
  image_sitter *st;

#ifdef DEBUG
  fprintf(stderr, "New image_sitter: %s\n", line);
#endif

  tk = token_string_new(line, ':');
  if (!token_string_has_more(tk)) {
    token_string_free(tk);
    return NULL;
  }

  st = g_new(image_sitter, 1);
  if ((st->geom = geometry_new(token_string_next(tk))) == NULL) {
    token_string_free(tk);
    g_free(st);
    return NULL;
  }

  if (!token_string_has_more(tk)) {
    token_string_free(tk);
    sitter_free((sitter*)st);
    return NULL;
  }

#ifdef DEBUG
  fprintf(stderr, "Loading image...\n");
#endif

  st->img = Imlib_load_image(XsitImlibData, token_string_next(tk));
  token_string_free(tk);

  if (st->img == NULL) {
    fprintf(stderr, "Unable to load image\n");
    exit(1);
  }

  Imlib_render(XsitImlibData,
	       st->img,
	       st->img->rgb_width,
	       st->img->rgb_height);

  st->super.w = st->img->rgb_width;
  st->super.h = st->img->rgb_height;

  st->super.type = IMAGE_SITTER;

  st->super.next_frame = image_sitter_next_frame;
  st->super.get_current_frame = image_sitter_get_current_frame;
  st->super.get_next_timeout = image_sitter_get_next_timeout;
  st->super.get_x = image_sitter_get_x;
  st->super.get_y = image_sitter_get_y;
  st->super.structure_notify = NULL;
  st->super.focus_notify = NULL;

  return (sitter*)st;
}

void image_sitter_free(sitter *s) {
  image_sitter *st = (image_sitter*)s;
  Imlib_destroy_image(XsitImlibData, st->img);
  geometry_free(st->geom);
  sitter_free(s);
}

static const char* _rcsid_imagesitter __attribute__((unused)) = "$Id: imagesitter.c,v 1.4 2003/08/04 14:40:16 jerome Exp $";
