
/*
    xsit - screen mate for X11
    Copyright (C) 2003 Jrme Laheurte <fraca7@free.fr>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* $Id: main.c,v 1.4 2003/08/04 14:40:16 jerome Exp $ */

#define _GNU_SOURCE

#include <stdio.h>
#include <glib.h>
#include <errno.h>

#include <Imlib.h>
#include <X11/Xlib.h>

#include "utils.h"
#include "window.h"

/* je sais, c'est pas b mais parfois c'est utile */

ImlibData *XsitImlibData;
Display *XsitDisplay;

gboolean xsit_idle(gpointer);

void usage() {
  printf("Usage: xsit -l\n");
  printf("  List available sitters.\n");
  printf("       xsit -s <name>\n");
  printf("  Use sitter <name>\n");
}

int main(int argc, char *argv[]) {
  FILE *rcfile;
  token_string *tk;
  gchar *bf, *rc, *sitter = NULL;
  gint list = 0, n;
  GMainLoop *loop;
  xsit_window *win;

  if ((argc != 2) && (argc != 3)) {
    usage();
    exit(1);
  }

  if (!strcmp(argv[1], "-42")) {
    printf("Is it 6x7 or 9x7 ?\n");
    exit(0); /* a le mrite */
  } else if (!strcmp(argv[1], "-l")) {
    list = 1;
  } else if ((strcmp(argv[1], "-s") != 0) || (argc != 3)) {
    usage();
    exit(1);
  }

  bf = g_get_home_dir();
  rc = g_strconcat(bf, "/.xsitrc", NULL);
  rcfile = fopen(rc, "r");
  if (rcfile == NULL) {
    perror("~/.xsitrc");
    exit(1);
  }
  g_free(rc);

  bf = NULL;
  n = 0;

  if (list != 0) {
    printf("Name\tDescription\n\n");
  }

  while (getline(&bf, &n, rcfile) != -1) {
    g_strstrip(bf);
    if ((bf[0] == 0) || (bf[0] == '#')) continue;

    tk = token_string_new(bf, ':');
    if (!token_string_has_more(tk)) {
      fprintf(stderr, "Parse error in ~/.xsitrc:\n%s\n", bf);
      exit(1);
    }

    if (list != 0) {
      printf("%s\t", token_string_next(tk));
      if (!token_string_has_more(tk)) {
	fprintf(stderr, "Parse error in ~/.xsitrc:\n%s\n", bf);
	exit(1);
      }
      printf("%s\n", token_string_next(tk));
    } else {
      sitter = token_string_next(tk);
      if (!strcmp(argv[2], sitter)) {
	sitter = g_strdup(bf);
	break;
      }
      sitter = NULL;
    }

    token_string_free(tk);
  }

  fclose(rcfile);
  g_free(bf);

  if (list != 0) exit(0);

  if (sitter == NULL) {
    fprintf(stderr, "No such sitter: %s\n", argv[2]);
    exit(1);
  }

#ifdef DEBUG
  fprintf(stderr, "Opening display\n");
#endif

  XsitDisplay = XOpenDisplay(NULL);

#ifdef DEBUG
  fprintf(stderr, "Initializing Imlib\n");
#endif

  XsitImlibData = Imlib_init(XsitDisplay);

  if (!token_string_has_more(tk)) {
    fprintf(stderr, "Parse error in ~/.xsitrc:\n%s\n", sitter);
  }
  (void)token_string_next(tk);

  if ((win = xsit_window_new(token_string_remain(tk))) == NULL) {
    fprintf(stderr, "Parse error in ~/.xsitrc:\n%s\n", sitter);
    exit(1);
  }

  g_idle_add(xsit_idle, win);

  loop = g_main_new(FALSE);
  g_main_run(loop);

  return 0; /* j'aime pas les warnings */
}

gboolean xsit_idle(gpointer w) {
  static XEvent xev;
  xsit_window *win = (xsit_window*)w;

  if (XCheckMaskEvent(XsitDisplay, StructureNotifyMask, &xev))
    xsit_window_handle_event(win);

  return TRUE;
}

static const char* _rcsid_main __attribute__((unused)) = "$Id: main.c,v 1.4 2003/08/04 14:40:16 jerome Exp $";
