
/*
    xsit - screen mate for X11
    Copyright (C) 2003 Jrme Laheurte <fraca7@free.fr>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* $Id: sitter.c,v 1.4 2003/08/04 14:40:16 jerome Exp $ */

#include "sitter.h"
#include "imagesitter.h"
#include "animationsitter.h"
#include "pluginsitter.h"
#include "utils.h"

void sitter_free_all(sitter *s) {
  switch (s->type) {
  case IMAGE_SITTER:
    image_sitter_free(s);
    break;
  case ANIMATION_SITTER:
    animation_sitter_free(s);
    break;
  case PLUGIN_SITTER:
    plugin_sitter_free(s);
    break;
  default:
    g_free(s);
  }
}

void sitter_free(sitter *s) {
  g_free(s);
}

sitter *sitter_new(gchar *str) {
  token_string *tk;
  gchar *bf;
  sitter *s;

#ifdef DEBUG
  fprintf(stderr, "New sitter: %s\n", str);
#endif

  tk = token_string_new(str, ':');
  if (!token_string_has_more(tk)) {
    token_string_free(tk);
    return NULL;
  }

  bf = token_string_next(tk);
  switch (bf[0]) {
  case 'i':
    s = image_sitter_new(token_string_remain(tk));
    break;
  case 'a':
    s = animation_sitter_new(token_string_remain(tk));
    break;
  case 'p':
    s = plugin_sitter_new(token_string_remain(tk));
    break;
  default:
    s = NULL;
  }

  token_string_free(tk);
  return s;
}

static const char* _rcsid_sitter __attribute__((unused)) = "$Id: sitter.c,v 1.4 2003/08/04 14:40:16 jerome Exp $";
