
/*
    xsit - screen mate for X11
    Copyright (C) 2003 Jrme Laheurte <fraca7@free.fr>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* $Id: window.c,v 1.4 2003/08/04 14:40:16 jerome Exp $ */

#include "window.h"

extern Display *XsitDisplay;
extern ImlibData *XsitImlibData;

gboolean xsit_window_timeout(gpointer);

Window get_toplevel(Window w) {
  Window pr, rr;
  Window *cr;
  unsigned int nc;

  if ((w == None) || (w == PointerRoot)) return None;

  pr = w;
  do {
    w = pr;
    if (!XQueryTree(XsitDisplay, w, &rr, &pr, &cr, &nc)) {
      w = None;
      break;
    }
    if (nc != 0) XFree(cr);
  } while (pr != rr);

  return w;
}

xsit_window *xsit_window_new(gchar *line) {
  xsit_window *w;
  XSetWindowAttributes xswa;

  w = g_new(xsit_window, 1);
  w->st = sitter_new(line);
  if (w->st == NULL) {
    g_free(w);
    return NULL;
  }

  w->focus = None;

  xswa.override_redirect = TRUE;
  w->win = XCreateWindow(XsitDisplay,
			 DefaultRootWindow(XsitDisplay),
			 0,
			 0,
			 w->st->w,
			 w->st->h,
			 0,
			 XsitImlibData->x.depth,
			 InputOutput,
			     XsitImlibData->x.visual,
			 CWOverrideRedirect,
			 &xswa);

  Imlib_apply_image(XsitImlibData,
		    (*(w->st->get_current_frame))(w->st),
		    w->win);

  w->counter = 0;
  w->timeout = (*(w->st->get_next_timeout))(w->st);

  g_timeout_add(100, xsit_window_timeout, (gpointer)w);
  return w;
}

void xsit_window_free(xsit_window *w) {
  sitter_free_all(w->st);
  XDestroyWindow(XsitDisplay, w->win);
  g_free(w);
}

gboolean xsit_window_check_focus(xsit_window *w) {
  Window win;
  int pipo;

  XGetInputFocus(XsitDisplay, &win, &pipo);
  win = get_toplevel(win);

  if (win != w->focus) {

#ifdef DEBUG
    fprintf(stderr, "Focus changed\n");
#endif

    if (w->focus != None) XSelectInput(XsitDisplay, w->focus, 0);
    if (win != None) XSelectInput(XsitDisplay, win, StructureNotifyMask);
    w->focus = win;

    if (w->st->focus_notify != NULL)
      (*(w->st->focus_notify))(w->st);

    return TRUE;
  }

  return FALSE;
}

void xsit_window_move(xsit_window *win) {
  int x, y;
  unsigned int w, h, bw, d;
  Window pipo;

  if (win->focus == None) {
    XUnmapWindow(XsitDisplay, win->win);
  } else {
    XGetGeometry(XsitDisplay, win->focus, &pipo, &x, &y, &w, &h, &bw, &d);
    XMoveWindow(XsitDisplay,
		win->win,
		(*(win->st->get_x))(win->st, x, y, w, h),
		(*(win->st->get_y))(win->st, x, y, w, h));
    XMapRaised(XsitDisplay, win->win);
  }
}

void xsit_window_handle_event(xsit_window *w) {
#ifdef DEBUG
  fprintf(stderr, "X Event\n");
#endif

  if (w->st->structure_notify != NULL)
    (*(w->st->structure_notify))(w->st);

  xsit_window_check_focus(w);
  xsit_window_move(w);
}

gboolean xsit_window_timeout(gpointer p) {
  gboolean r = FALSE;
  xsit_window *w = (xsit_window*)p;
  w->counter++;
  r = r || (xsit_window_check_focus(w));
  if (w->timeout != 0) {
    if (w->counter > w->timeout) {
      r = TRUE;
      w->counter = 0;
      w->timeout = (*(w->st->get_next_timeout))(w->st);
      (*(w->st->next_frame))(w->st);
      Imlib_apply_image(XsitImlibData,
			(*(w->st->get_current_frame))(w->st),
			w->win);
    }
  }

  if (r)
    xsit_window_move(w);

  return TRUE;
}

static const char* _rcsid_window __attribute__((unused)) = "$Id: window.c,v 1.4 2003/08/04 14:40:16 jerome Exp $";
